@@ -12,10 +12,13 @@ tnk_vocs = VOCS(
         "objectives": {"y1": "MINIMIZE", "y2": "MINIMIZE"},
         "constraints": {"c1": ["GREATER_THAN", 0], "c2": ["LESS_THAN", 0.5]},
         "constants": {"a": "dummy_constant"},
-        # 'linked_variables': {'x9': 'x1'}
     }
 )
 
+# With constraints, objectives are below ~1.05
+# https://pymoo.org/problems/multi/tnk.html
+tnk_reference_point = {"y1": 2.5, "y2": 2.5}
+
 
 # Pure number version
 def TNK(individual):
@@ -40,14 +43,13 @@ def evaluate_TNK(inputs: Dict, sleep=0, random_sleep=0, raise_probability=0, **p
     time.sleep(r * random_sleep * 2)  # Average should be random_sleep
 
     if r < raise_probability:
-        raise ValueError("intentioal TNK crash")
+        raise ValueError("intentional TNK crash")
 
     outputs = {
         "y1": objectives[0],
         "y2": objectives[1],
         "c1": constraints[0],
         "c2": constraints[1],
-        "some_array": np.array([1, 2, 3]),
     }
 
     return outputs
